@@ -909,13 +909,16 @@ class DuPanShareList:
         session.headers["Referer"] = url
 
     def __iter__(self, /) -> Iterator[dict]:
-        dq = deque("/")
+        dq: deque[tuple[str, str]] = deque()
         get, put = dq.popleft, dq.append
+        put(("", ""))
         while dq:
-            for file in self.iterdir(get()):
+            dir, dir_relpath = get()
+            for file in self.iterdir(dir):
+                relpath = file["relpath"] = joinpath(dir_relpath, file["server_filename"])
                 yield file
                 if file["isdir"]:
-                    put(file["path"])
+                    put((file["path"], relpath))
 
     @staticmethod
     def _extract_from_url(url: str, /) -> tuple[str, str]:
@@ -1018,21 +1021,18 @@ class DuPanShareList:
                 else:
                     raise OSError(json)
 
-    def iterdir(self, /, dir="/") -> Iterator[dict]:
+    def iterdir(self, /, dir: str = "/", page: int = 1, num: int = 0) -> Iterator[dict]:
         if dir in ("", "/"):
-            yield from self.list_index()
+            data = self.list_index()
+            if num <= 0 or page <= 0:
+                yield from data
+            else:
+                yield from data[(page-1)*num:page*num]
             return
         if not hasattr(self, "share_uk"):
             self.list_index()
         if not dir.startswith("/"):
             dir = self.root + "/" + dir
-            start_inx = len(dir) + 1
-        elif dir == self.root or dir.startswith(self.root + "/"):
-            start_inx = len(self.root) + 1
-        elif dir == self.root2 or dir.startswith(self.root2 + "/"):
-            start_inx = len(self.root2) + 1
-        else:
-            raise FileNotFoundError(errno.ENOENT, dir)
         api = "https://pan.baidu.com/share/list"
         params = {
             "uk": self.share_uk, 
@@ -1047,14 +1047,21 @@ class DuPanShareList:
             "dir": dir, 
         }
         get = self.session.get
-        while True:
-            ls = Error.check(get(api, params=params).json())["list"]
-            for file in ls:
-                file["relpath"] = file["path"][start_inx:]
-            yield from ls
-            if len(ls) < 100:
-                return
-            params["page"] += 1
+        if num <= 0 or page <= 0:
+            if num > 0:
+                params["num"] = num
+            else:
+                num = params["num"]
+            while True:
+                ls = Error.check(get(api, params=params).json())["list"]
+                yield from ls
+                if len(ls) < num:
+                    break
+                params["page"] += 1
+        else:
+            params["page"] = page
+            params["num"] = num
+            yield from Error.check(get(api, params=params).json())["list"]
 
     def list_index(self, /, try_times: int = 5) -> list[dict]:
         url = self.url
@@ -1099,60 +1106,11 @@ class DuPanShareList:
                     return file_list
         raise RuntimeError("too many attempts")
 
-    def listdir(self, /, dir="/", page=1, num=0) -> list[str]:
-        return [a["server_filename"] for a in self.listdir_attr(dir, page, num)]
+    def listdir(self, /, dir: str = "/", page: int = 1, num: int = 0) -> list[str]:
+        return [attr["server_filename"] for attr in self.iterdir(dir, page, num)]
 
-    def listdir_attr(self, /, dir="/", page=1, num=0) -> list[dict]:
-        if dir in ("", "/"):
-            data = self.list_index()
-            if num <= 0:
-                return data
-            if page < 1:
-                page = 1
-            return data[(page-1)*num:page*num]
-        if not hasattr(self, "share_uk"):
-            self.list_index()
-        if not dir.startswith("/"):
-            dir = self.root + "/" + dir
-            start_inx = len(dir) + 1
-        elif dir == self.root or dir.startswith(self.root + "/"):
-            start_inx = len(self.root) + 1
-        elif dir == self.root2 or dir.startswith(self.root2 + "/"):
-            start_inx = len(self.root2) + 1
-        else:
-            raise FileNotFoundError(errno.ENOENT, dir)
-        api = "https://pan.baidu.com/share/list"
-        params = {
-            "uk": self.share_uk, 
-            "shareid": self.share_id, 
-            "order": "other", 
-            "desc": 1, 
-            "showempty": 0, 
-            "clienttype": 0, 
-            "web": 1, 
-            "page": 1, 
-            "num": 100, 
-            "dir": dir, 
-        }
-        session = self.session
-        if num <= 0:
-            ls_all = []
-            while True:
-                ls = Error.check(session.get(api, params=params).json())["list"]
-                ls_all.extend(ls)
-                if len(ls) < 100:
-                    for file in ls_all:
-                        file["relpath"] = file["path"][start_inx:]
-                    return ls_all
-                params["page"] += 1
-        if page > 0:
-            params["page"] = page
-        if num < 100:
-            params["num"] = 100
-        ls = Error.check(session.get(api, params=params).json())["list"]
-        for file in ls:
-            file["relpath"] = file["path"][start_inx:]
-        return ls
+    def listdir_attr(self, /, dir: str = "/", page: int = 1, num: int = 0) -> list[dict]:
+        return list(self.iterdir(dir, page, num))
 
 
 class DuPanPath:
@@ -2301,9 +2259,16 @@ class DuPanFileSystem:
         path = cast(str, path)
         if path == "/":
             return "/"
-        if not exist_ok and self.exists(path, _check=False):
-            raise FileExistsError(errno.EEXIST, path)
-        Error.check(self.client.makedir(path))
+        try:
+            attr = self.attr(path, _check=False)
+        except FileNotFoundError:
+            if exist_ok:
+                if not attr["isdir"]:
+                    raise NotADirectoryError(errno.ENOTDIR, path)
+            else:
+                raise FileExistsError(errno.EEXIST, path)
+        else:
+            return Error.check(self.client.makedir(path))["path"]
         return path
 
     def mkdir(
@@ -2685,6 +2650,38 @@ class DuPanFileSystem:
             return self.upload(BytesIO(), path, _check=False)
         return path
 
+    # TODO: 确保转存后也保持目录结构
+    def transfer(
+        self, 
+        /, 
+        url: str, 
+        password: str = "", 
+        fsidlist: None | list[int] = None, 
+        save_dir: str = "/", 
+    ) -> dict:
+        """转存文件到百度网盘
+
+        :param url: 分享链接
+        :param password: 密码（如果链接中包含密码，可以不传）
+        :param fsidlist: 待转存的文件 id 列表
+        :param save_dir: 存储到这个文件夹，默认是 /，也就是网盘根目录（请提前建立这个文件夹）
+
+        :return: 转存接口返回到 JSON 信息
+        """
+        share_list = DuPanShareList(url, password)
+        if not fsidlist:
+            fsidlist = [f["fs_id"] for f in share_list.list_index()]
+        return self.client.transfer(
+            share_list.url, 
+            params={
+                "shareid": share_list.share_id, 
+                "from": share_list.share_uk, 
+                "sekey": share_list.randsk, 
+                "ondup": "overwrite", 
+            }, 
+            data = {"fsidlist": fsidlist, "path": save_dir}, 
+        )
+
     def upload(
         self, 
         /, 
